#!/bin/bash 
##############################################################################
# MegaTRiAL 1.1 by Turranius - http://www.grandis.nu/glftpd/
# This is the shell version of the script. It should have come with a tcl as 
# well to trigger the script from IRC. 
# I suck at tcl, so I'll stay with shell scripts until I.. dont suck anymore.
#
# This script will display users on trial this week, and those up for trial
# next week, when someone executes the !trials command from IRC.
# It will also show groups on trial this week, and next.
# As with version 1.1, it does the same for groups.
#
# Also new in 1.1 is that it handles up to 9 stat sections automatically.
# It will do this automatically if they find them though.
#
# This script will NOT delete user OR change anything whatsoever. It is 
# simply a way of viewing how folks are doing.
# I dont take any responsibility if it ruins anything for you though :))
##############################################################################
# Installation:
# Put this script in /glftpd/bin. Make it executable, and make sure the user
# who runs the bot has rights to execute it too.
# Put the tcl script in your bots /scripts folder and load it in the bots
# config file. Dont forget to rehash the bot.
# Edit the paths and limits below so it suits your system / needs.
# Edit the tcl if you wish to change the trigger. Open it for more info on
# that.
# You can enable or disable any of the 4 functions in this script.
# For instance, if you do not have any group trial, or dont like the feature,
# set GROUPTRIAL and NEXTGROUP to "FALSE" below.
#
# There is also a function called PASSEARLY. Setting this to TRUE will
# show how much next weeks users have uploaded, and if they passed the limit.
# Setting it to FALSE will simply show the names of next weeks trial users.
# Some people thought it might be confusing if it said PASSED when they are
# not trial yet. I just think its a neat feature to show it.
#
# If you have no groups/users on trial, it will not display anything about that
# but I do recomend you set those you dont use to FALSE, since it will execute
# much faster then. So, if you dont want group trials, set them to false, even 
# though it wont show anything about groups if theyre is none set for trial.
#
# Now then. How to set the users or groups on trial?
# For users, there should be 2 groups. TRiAL & TRiAL-2. 
# TRiAL means they are on trial this week.
# TRiAL-2 means they are up for trial next week.
# A user should NEVER be in both TRiAL and TRiAL-2 at the same time!
#
# For groups, use the grpnfo function to change their nfo to either
# TRiAL or TRiAL-2.
# As with users, a group nfo of TRiAL means they are on trial this week and
# TRiAL-2 means next week.
#
# You can execute this script from shell to test it. If it works from
# shell, it should work from IRC as well.
#
# I recomend you run this in crontab at 11:55PM on Saturday and pipe
# the output to a logfile somewhere. That way you dont have to sit next
# to the computer before the stats reset. Just check the logfile when you
# want to. Dont forget to change any TRiAL-2 users or groups to TRiAL 
# once a new week has started though :)
#
# You may change and release this script the best you want. All I ask is that
# you shoot me a msg if you use and like it :)
################################################################################################
# Changelog:                
# 1.1 : Add: Now allows you to set a GROUPLIMIT so you can check directly
#            if groups have uploaded the required limit. 
#       Add: Added DIREXCLUDE. Exclude folders in users folder, and users.
#       Add: Now handles both Weekly and Monthly trials.
#       Add: If a user has passed, instead of just saying PASSED, it will show how much
#            the user uploaded, like PASSED(1600MB).
#       Add: Now handles up to 9 stat sections in all 4 trial modes. This is done
#            automatically, so no need to bother about it. It will simply add them up.
#       Fix: If PASSEARLY is FALSE, it wont check how much those people uploaded anymore.
#            Should be a little bit faster then if you had it on FALSE with v1.0.
#
# 1.0 : Initial release.
################################################################################################
# Settings: 

LIMIT=1499                              # MB limit for single users -1 (if trial is 1500MB, set it to 1499).
GROUPLIMIT=7500                         # MB limit for groups to pass -1 (if trial is 7500MB, set it to 7499).
USERSLOCATION=/glftpd/ftp-data/users    # Userfiles location if you have WEEKLY or NEXTWEEK on TRUE.
GROUPFILE=/glftpd/etc/group             # groups file if you have GROUPTRIAL or NEXTGROUP on TRUE.
PERIOD=W                                # W = Weekly / M = Monthly.
THISPERIOD=TRUE                         # TRUE/FALSE. Announce Single users on trial this period.
NEXTPERIOD=TRUE                         # TRUE/FALSE. Announce Single traders up for trial next period.
GROUPTRIAL=TRUE                         # TRUE/FALSE. Announce Groups on trial for this week.
NEXTGROUP=TRUE                          # TRUE/FALSE. Announce Groups on trials for next week.
PASSEARLY=TRUE                          # TRUE/FALSE. Can users or groups pass trial when they are in TRiAL-2?
                                        # TRUE = Show how much they uploaded. FALSE = Just show the name.

DIREXCLUDE='default.user|backup|disabled' # If you have folders in your users folder, add them here.
                                          # Users can also be added (But thats little use).
                                          # Separate folders with | like 'folder1|folder2|user1'
                                          # Do not remove the ' at the beginning and end.
                                          # If you have a group on trial, but one person in that group
                                          # should not count, add him here too.

###################################################################################################################
# No Change Below Here Needed                                                                                     #
###################################################################################################################

FOUNDONE="NO"

if [ "$PERIOD" != "W" ]; then
  if [ "$PERIOD" != "M" ]; then
    echo "Period must be W or M".
    exit 1
  fi
fi

if [ "$PERIOD" = "W" ]; then
  DUR="WKUP"
  DURNAME="Week"
else
  DUR="MONTHUP"
  DURNAME="Month"
fi

##################################################################################
# Single users up for trial this period.

if [ "$THISPERIOD" = "TRUE" ]; then
  TRIALSAID="NO"

  cd $USERSLOCATION
  for i in `ls -f -A | egrep -v $DIREXCLUDE`; do
    GROUP=`grep GROUP $i | awk -F" " '{print $2}'`
    GROUP="$(echo $GROUP | grep -w TRiAL | egrep -v 'TRiAL-2' | sort -k 2,2 -n -r)"
    if [ "$GROUP" != "" ]; then
      UP=`grep -w $DUR $i | awk -F" " '{print $3}'`
      if [ "$UP" = "" ]; then
        UP="0"
      fi

      ## Check for other stat sections.
      UP2="$( grep -w $DUR $i | awk -F" " '{print $6}')"
      if [ "$UP2" != "" ]; then
        UP3="$(grep -w $DUR $i | awk -F" " '{print $9}')"
        if [ "$UP3" != "" ]; then
          UP4="$(grep -w $DUR $i | awk -F" " '{print $12}')"
          if [ "$UP4" != "" ]; then
            UP5="$(grep -w $DUR $i | awk -F" " '{print $15}')"
            if [ "$UP5" != "" ]; then
              UP6="$(grep -w $DUR $i | awk -F" " '{print $18}')"
              if [ "$UP6" != "" ]; then
                UP7="$(grep -w $DUR $i | awk -F" " '{print $21}')"
                if [ "$UP7" != "" ]; then
                  UP8="$(grep -w $DUR $i | awk -F" " '{print $24}')"
                  if [ "$UP8" != "" ]; then
                    UP9="$(grep -w $DUR $i | awk -F" " '{print $27}')"
                    if [ "$UP9" != "" ]; then
                      UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" \+ "$UP9" )"
                    else
                      UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" )"
                    fi
                  else
                    UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" )"
                  fi
                else
                  UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" )"
                fi
              else
                UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" )"
              fi
            else
              UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" )"
            fi
          else
            UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" )"
          fi
        else
          UP="$( expr "$UP" \+ "$UP2" )"
        fi
      fi
          
      UP="$( expr "$UP" \/ "1024" )"

      if [ "$TRIALSAID" != "YES" ]; then
        echo "TRiAL Users =" | awk '{ printf($1" "$2" "$3" ")}'
        TRIALSAID="YES"
      fi

      if [ "$UP" -gt "$LIMIT" ]; then
        UP="PASSED($UP"MB")"
      else
        UP="$UP"MB""
      fi
      FOUNDONE="YES"

      echo $i $UP | awk '{ printf($1"/"$2" - ")}'
    fi
  done
fi

##################################################################################
# Single user up for trial next period.

if [ "$NEXTPERIOD" = "TRUE" ]; then
  TRIALSAID="NO"
  UP=""

  cd $USERSLOCATION
  for i in `ls -f -A | egrep -v $DIREXCLUDE`; do
    GROUP=`grep GROUP $i | awk -F" " '{print $2}'`
    GROUP="$(echo $GROUP | grep -w TRiAL-2 | sort -k 2,2 -n -r)"
    if [ "$GROUP" != "" ]; then
      if [ "$PASSEARLY" = "TRUE" ]; then
        UP=`grep -w $DUR $i | awk -F" " '{print $3}'`
        if [ "$UP" = "" ]; then
          UP="0"
        fi

        ## Check for other stat sections.
        UP2="$( grep -w $DUR $i | awk -F" " '{print $6}')"
        if [ "$UP2" != "" ]; then
          UP3="$(grep -w $DUR $i | awk -F" " '{print $9}')"
          if [ "$UP3" != "" ]; then
            UP4="$(grep -w $DUR $i | awk -F" " '{print $12}')"
            if [ "$UP4" != "" ]; then
              UP5="$(grep -w $DUR $i | awk -F" " '{print $15}')"
              if [ "$UP5" != "" ]; then
                UP6="$(grep -w $DUR $i | awk -F" " '{print $18}')"
                if [ "$UP6" != "" ]; then
                  UP7="$(grep -w $DUR $i | awk -F" " '{print $21}')"
                  if [ "$UP7" != "" ]; then
                    UP8="$(grep -w $DUR $i | awk -F" " '{print $24}')"
                    if [ "$UP8" != "" ]; then
                      UP9="$(grep -w $DUR $i | awk -F" " '{print $27}')"
                      if [ "$UP9" != "" ]; then
                        UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" \+ "$UP9" )"
                      else
                        UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" )"
                      fi
                    else
                      UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" )"
                    fi
                  else
                    UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" )"
                  fi
                else
                  UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" )"
                fi
              else
                UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" )"
              fi
            else
              UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" )"
            fi
          else
            UP="$( expr "$UP" \+ "$UP2" )"
          fi
        fi
          
        UP="$( expr "$UP" \/ "1024" )"

        if [ "$UP" -gt "$LIMIT" ]; then
          UP="Passed($UP"MB")"
        else
          UP="$UP"MB""
        fi
      fi

      if [ "$TRIALSAID" != "YES" ]; then
        echo "Users Next $DURNAME =" | awk '{ printf($1" "$2" "$3" "$4" ")}'
        TRIALSAID="YES"
      fi

      FOUNDONE="YES"
      
      if [ "$PASSEARLY" = "TRUE" ]; then
        echo $i $UP | awk '{ printf($1"/"$2" - ")}'
      else
        echo $i | awk '{ printf($1" - ")}'
      fi
    fi
  done
fi

##################################################################################
# Groups doing trial this period.

if [ "$GROUPTRIAL" = "TRUE" ]; then
  TRIALSAID="NO"
  cd $USERSLOCATION

  for i in `cat $GROUPFILE`; do
    TRIALG="$(echo $i | awk -F":" '{print $2}')"
    if [ "$TRIALG" = "TRiAL" ]; then
      NAME="$(echo $i | awk -F":" '{print $1}')"
      if [ "$TRIALSAID" != "YES" ]; then
        echo "TRiAL Groups =" | awk '{ printf($1" "$2" "$3" ")}'
        TRIALSAID="YES"
      fi
      for u in `ls -f -A | egrep -v $DIREXCLUDE`; do
        if [ "$LAST" != $i ]; then
          TOTAL="0"
        fi
        GROUP="$(cat $u | grep -w GROUP | awk -F" " '{print $2}')"
        INIT="$( echo $GROUP | grep -w $NAME )"
        if [ "$INIT" != "" ]; then
          UP="$(cat $u | grep -w $DUR | awk -F" " '{print $3}')"

          ## Check for other stat sections.
          UP2="$( grep -w $DUR $u | awk -F" " '{print $6}')"
          if [ "$UP2" != "" ]; then
            UP3="$(grep -w $DUR $u | awk -F" " '{print $9}')"
            if [ "$UP3" != "" ]; then
              UP4="$(grep -w $DUR $u | awk -F" " '{print $12}')"
              if [ "$UP4" != "" ]; then
                UP5="$(grep -w $DUR $u | awk -F" " '{print $15}')"
                if [ "$UP5" != "" ]; then
                  UP6="$(grep -w $DUR $u | awk -F" " '{print $18}')"
                  if [ "$UP6" != "" ]; then
                    UP7="$(grep -w $DUR $u | awk -F" " '{print $21}')"
                    if [ "$UP7" != "" ]; then
                      UP8="$(grep -w $DUR $u | awk -F" " '{print $24}')"
                      if [ "$UP8" != "" ]; then
                        UP9="$(grep -w $DUR $u | awk -F" " '{print $27}')"
                        if [ "$UP9" != "" ]; then
                          UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" \+ "$UP9" )"
                        else
                          UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" )"
                        fi
                      else
                        UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" )"
                      fi
                    else
                      UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" )"
                    fi
                  else
                    UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" )"
                  fi
                else
                  UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" )"
                fi
              else
                UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" )"
              fi
            else
              UP="$( expr "$UP" \+ "$UP2" )"
            fi
          fi

          UP="$(expr $UP \/ 1024)"
          TOTAL="$(expr $UP \+ $TOTAL)"
        fi
        LAST=$i
      done
      
      if [ "$TOTAL" -gt "$GROUPLIMIT" ]; then
        TOTAL="Passed("$TOTAL"MB)"
      else
        TOTAL="$TOTAL"MB""
      fi
      FOUNDONE="YES"

      echo "$NAME $TOTAL" | awk '{ printf($1" "$2" - ")}'
    fi
  done
fi

##################################################################################
# Groups up for trial next period.

if [ "$NEXTGROUP" = "TRUE" ]; then
  TRIALSAID="NO"
  TOTAL="0"
  NAME=""
  LAST=""

  for i in `cat $GROUPFILE`; do
    TRIALG="$(echo $i | awk -F":" '{print $2}')"
    if [ "$TRIALG" = "TRiAL-2" ]; then
      NAME="$(echo $i | awk -F":" '{print $1}')"
      if [ "$TRIALSAID" != "YES" ]; then
        echo "Groups next $DURNAME =" | awk '{ printf($1" "$2" "$3" "$4" ")}'
        TRIALSAID="YES"
      fi

      if [ "$PASSEARLY" = "TRUE" ]; then
        for u in `ls -f -A | egrep -v $DIREXCLUDE`; do
          if [ "$LAST" != $i ]; then
            TOTAL="0"
          fi
          GROUP="$(cat $u | grep -w GROUP | awk -F" " '{print $2}')"
          INIT="$( echo $GROUP | grep -w $NAME )"
          if [ "$INIT" != "" ]; then
            UP="$(cat $u | grep -w $DUR | awk -F" " '{print $3}')"

            ## Check for other stat sections.
            UP2="$( grep -w $DUR $u | awk -F" " '{print $6}')"
            if [ "$UP2" != "" ]; then
              UP3="$(grep -w $DUR $u | awk -F" " '{print $9}')"
              if [ "$UP3" != "" ]; then
                UP4="$(grep -w $DUR $u | awk -F" " '{print $12}')"
                if [ "$UP4" != "" ]; then
                  UP5="$(grep -w $DUR $u | awk -F" " '{print $15}')"
                  if [ "$UP5" != "" ]; then
                    UP6="$(grep -w $DUR $u | awk -F" " '{print $18}')"
                    if [ "$UP6" != "" ]; then
                      UP7="$(grep -w $DUR $u | awk -F" " '{print $21}')"
                      if [ "$UP7" != "" ]; then
                        UP8="$(grep -w $DUR $u | awk -F" " '{print $24}')"
                        if [ "$UP8" != "" ]; then
                          UP9="$(grep -w $DUR $u | awk -F" " '{print $27}')"
                          if [ "$UP9" != "" ]; then
                            UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" \+ "$UP9" )"
                          else
                            UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" \+ "$UP8" )"
                          fi
                        else
                          UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" \+ "$UP7" )"
                        fi
                      else
                        UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" \+ "$UP6" )"
                      fi
                    else
                      UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" \+ "$UP5" )"
                    fi
                  else
                    UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" \+ "$UP4" )"
                  fi
                else
                  UP="$( expr "$UP" \+ "$UP2" \+ "$UP3" )"
                fi
              else
                UP="$( expr "$UP" \+ "$UP2" )"
              fi
            fi
          
            UP="$(expr $UP \/ 1024)"
            TOTAL="$(expr $UP \+ $TOTAL)"
          fi
          LAST=$i
        done
      
        if [ "$TOTAL" -gt "$GROUPLIMIT" ]; then
          TOTAL="Passed("$TOTAL"MB)"
        else
          TOTAL="$TOTAL"MB""
        fi
      fi
      FOUNDONE="YES"

      if [ "$PASSEARLY" = "TRUE" ]; then
        echo "$NAME $TOTAL" | awk '{ printf($1" "$2" - ")}'
      else
        echo "$NAME" | awk '{ printf($1" - ")}'
      fi
    fi
  done
fi

##################################################################################
# If neither users or group on trial are found, this is displayed.

if [ "$FOUNDONE" != "YES" ]; then
  echo "No users or groups on trial from what I can see..."
  exit 0
fi

exit 0